@@ -1,9 +1,6 @@
 package Log::WarnDie;
 
-# Make sure we have version info for this module
-# Be strict from now on
-
-$VERSION = '0.05';
+use warnings;
 use strict;
 
 # Make sure we have the modules that we need
@@ -15,14 +12,72 @@ use Scalar::Util qw(blessed);
 # The (original) error output handle
 # Reference to the previous parameters sent
 
-my $DISPATCHER;
-my $STDERR;
-my $LAST;
+our $DISPATCHER;
+our $STDERR;
+our $LAST;
 
 # Old settings of standard Perl logging mechanisms
 
-my $WARN;
-my $DIE;
+our $WARN;
+our $DIE;
+
+=head1 VERSION
+
+Version 0.06
+
+=head1 NAME
+
+Log::WarnDie - Log standard Perl warnings and errors on a log handler
+
+=head1 SYNOPSIS
+
+ use Log::WarnDie; # install to be used later
+
+ my $dispatcher = Log::Dispatch->new;       # can be any dispatcher!
+ $dispatcher->add( Log::Dispatch::Foo->new( # whatever output you like
+  name      => 'foo',
+  min_level => 'info',
+ ) );
+
+ use Log::WarnDie $dispatcher; # activate later
+
+ Log::WarnDie->dispatcher( $dispatcher ); # same
+
+ warn "This is a warning";       # now also dispatched
+ die "Sorry it didn't work out"; # now also dispatched
+
+ no Log::WarnDie; # deactivate later
+
+ Log::WarnDie->dispatcher( undef ); # same
+
+ warn "This is a warning"; # no longer dispatched
+ die "Sorry it didn't work out"; # no longer dispatched
+
+=head1 DESCRIPTION
+
+The "Log::WarnDie" module offers a logging alternative for standard
+Perl core functions.  This allows you to use the features of e.g.
+L<Log::Dispatch> or L<Log::Log4perl> B<without> having to make extensive
+changes to your source code.
+
+When loaded, it installs a __WARN__ and __DIE__ handler and intercepts any
+output to STDERR.  It also takes over the messaging functions of L<Carp>.
+Without being further activated, the standard Perl logging functions continue
+to be executed: e.g. if you expect warnings to appear on STDERR, they will.
+
+Then, when necessary, you can activate actual logging through e.g.
+Log::Dispatch by installing a log dispatcher.  From then on, any warn, die,
+carp, croak, cluck, confess or print to the STDERR handle,  will be logged
+using the Log::Dispatch logging dispatcher.  Logging can be disabled and
+enabled at any time for critical sections of code.
+
+=cut
+
+our $VERSION = '0.06';
+
+=head1 SUBROUTINES/METHODS
+
+=cut
 
 #---------------------------------------------------------------------------
 
@@ -64,6 +119,31 @@ sub PRINT {
     print $STDERR @_;
 } #PRINT
 
+#---------------------------------------------------------------------------
+# PRINTF
+#
+# Called whenever something is printed on STDERR using printf
+#
+#  IN: 1 blessed object returned by TIEHANDLE
+#      2..N whatever was needed to be printed
+
+sub PRINTF {
+
+# Lose the object
+# If there is a dispatcher
+#  Put it in the log handler if not the same as last time
+#  Reset the flag
+# Make sure it appears on the original STDERR as well
+
+    shift;
+    if ($DISPATCHER) {
+        $DISPATCHER->error( @_ )
+         unless $LAST and @$LAST == @_ and join( '',@$LAST ) eq join( '',@_ );
+        undef $LAST;
+    }
+    printf $STDERR @_;
+} #PRINT
+
 #---------------------------------------------------------------------------
 # At compile time
 #  Create new handle
@@ -87,7 +167,11 @@ BEGIN {
     $SIG{__WARN__} = sub {
         if ($DISPATCHER) {
             $LAST = \@_;
-            $DISPATCHER->warning( @_ );
+	    if(ref($DISPATCHER) =~ /^Log::Log4perl/) {
+		$DISPATCHER->warn( @_ );
+	    } else {
+		    $DISPATCHER->warning( @_ );
+	   }
         }
         $WARN ? $WARN->( @_ ) : CORE::warn( @_ );
     };
@@ -100,10 +184,15 @@ BEGIN {
 #   Executes the standard system die() or whatever was there before
 
     $DIE = $SIG{__DIE__};
+    ::diag(ref($DISPATCHER));
     $SIG{__DIE__} = sub {
         if ($DISPATCHER) {
             $LAST = \@_;
-            $DISPATCHER->critical( @_ );
+	    if(ref($DISPATCHER) =~ /^Log::Log4perl/) {
+		$DISPATCHER->fatal( @_ );
+	    } else {
+		    $DISPATCHER->critical( @_ );
+	   }
         }
         $DIE ? $DIE->( @_ ) : CORE::die( @_ );
     };
@@ -122,13 +211,16 @@ BEGIN {
 # Class methods
 
 #---------------------------------------------------------------------------
-# dispatcher
-#
-# Set and/or return the current dispatcher
-#
-#  IN: 1 class (ignored)
-#      2 new dispatcher (optional)
-# OUT: 1 current dispatcher
+
+=head2 dispatcher
+
+Class method to set and/or return the current dispatcher
+
+ IN: 1 class (ignored)
+     2 new dispatcher (optional)
+ OUT: 1 current dispatcher
+
+=cut
 
 sub dispatcher {
 
@@ -187,56 +279,6 @@ sub unimport { import( undef ) } #unimport
 
 __END__
 
-=head1 NAME
-
-Log::WarnDie - Log standard Perl warnings and errors on a log handler
-
-=head1 SYNOPSIS
-
- use Log::WarnDie; # install to be used later
-
- my $dispatcher = Log::Dispatch->new;       # can be any dispatcher!
- $dispatcher->add( Log::Dispatch::Foo->new( # whatever output you like
-  name      => 'foo',
-  min_level => 'info',
- ) );
-
- use Log::WarnDie $dispatcher; # activate later
-
- Log::WarnDie->dispatcher( $dispatcher ); # same
-
- warn "This is a warning";       # now also dispatched
- die "Sorry it didn't work out"; # now also dispatched
-
- no Log::WarnDie; # deactivate later
-
- Log::WarnDie->dispatcher( undef ); # same
-
- warn "This is a warning"; # no longer dispatched
- die "Sorry it didn't work out"; # no longer dispatched
-
-=head1 VERSION
-
-This documentation describes version 0.05.
-
-=head1 DESCRIPTION
-
-The "Log::WarnDie" module offers a logging alternative for standard
-Perl core functions.  This allows you to use the features of e.g.
-L<Log::Dispatch> or L<Log::Log4Perl> B<without> having to make extensive
-changes to your source code.
-
-When loaded, it installs a __WARN__ and __DIE__ handler and intercepts any
-output to STDERR.  It also takes over the messaging functions of L<Carp>.
-Without being further activated, the standard Perl logging functions continue
-to be executed: e.g. if you expect warnings to appear on STDERR, they will.
-
-Then, when necessary, you can activate actual logging through e.g.
-Log::Dispatch by installing a log dispatcher.  From then on, any warn, die,
-carp, croak, cluck, confess or print to the STDERR handle,  will be logged
-using the Log::Dispatch logging dispatcher.  Logging can be disabled and
-enabled at any time for critical sections of code.
-
 =head1 LOG LEVELS
 
 The following log levels are used:
@@ -270,9 +312,11 @@ the real use of this module only comes into view when such a module B<is>
 installed.  Please note that for testing this module, you will need the
 L<Log::Dispatch::Buffer> module to also be available.
 
-An alternate logger may be L<Log::Log4Perl>, although this has not been tested
+An alternate logger may be L<Log::Log4perl>, although this has not been tested
 by the author.  Any object that provides a C<warning>, C<error> and C<critical>
 method, will operate with this module.
+Log4perl does not the message 'critical', so it will not work.
+A wishlist request has been sent (RT121065).
 
 =head2 eval
 
@@ -290,9 +334,17 @@ Unfortunately there is no automatic way to do that for you.
 
 =head1 AUTHOR
 
-Elizabeth Mattijsen, <liz@dijkmat.nl>.
+Elizabeth Mattijsen, <liz@dijkmat.nl>
 
-Please report bugs to <perlbugs@dijkmat.nl>.
+Maintained by Nigel Horne, C<< <njh at bandsman.co.uk> >>
+
+=head1 BUGS
+
+Please report any bugs or feature requests to C<bug-log-warndie at rt.cpan.org>,
+or through the web interface at
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Log-WarnDie>.
+I will be notified, and then you'll
+automatically be notified of progress on your bug as I make changes.
 
 =head1 COPYRIGHT
 
@@ -300,4 +352,6 @@ Copyright (c) 2004, 2007 Elizabeth Mattijsen <liz@dijkmat.nl>. All rights
 reserved.  This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
+Portions of versions 0.06 onwards, Copyright 2017 Nigel Horne
+
 =cut
